"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const isIterable_1 = require("itiriri/utils/isIterable");
const concat_1 = require("./iterators/concat");
const distinct_1 = require("./iterators/distinct");
const exclude_1 = require("./iterators/exclude");
const filter_1 = require("./iterators/filter");
const flat_1 = require("./iterators/flat");
const groupJoin_1 = require("./iterators/groupJoin");
const intersect_1 = require("./iterators/intersect");
const join_1 = require("./iterators/join");
const leftJoin_1 = require("./iterators/leftJoin");
const map_1 = require("./iterators/map");
const skip_1 = require("./iterators/skip");
const slice_1 = require("./iterators/slice");
const take_1 = require("./iterators/take");
const average_1 = require("./reducers/average");
const awaitAll_1 = require("./reducers/awaitAll");
const first_1 = require("./reducers/first");
const forEach_1 = require("./reducers/forEach");
const indexOf_1 = require("./reducers/indexOf");
const last_1 = require("./reducers/last");
const lastIndexOf_1 = require("./reducers/lastIndexOf");
const length_1 = require("./reducers/length");
const max_1 = require("./reducers/max");
const min_1 = require("./reducers/min");
const nth_1 = require("./reducers/nth");
const reduce_1 = require("./reducers/reduce");
const sum_1 = require("./reducers/sum");
const asyncIterator_1 = require("./utils/asyncIterator");
const isAsyncIterable_1 = require("./utils/isAsyncIterable");
function itiririAsync(source) {
    return new ItiririAsync(source);
}
exports.default = itiririAsync;
class ItiririAsync {
    constructor(source) {
        this.source = source;
    }
    [Symbol.asyncIterator]() {
        return asyncIterator_1.asyncIterator(this.source);
    }
    entries() {
        return new ItiririAsync(map_1.map(this.source, (elem, idx) => [idx, elem]));
    }
    keys() {
        return new ItiririAsync(map_1.map(this.source, (_, idx) => idx));
    }
    values() {
        return new ItiririAsync(this.source);
    }
    forEach(action) {
        return forEach_1.forEach(this.source, action);
    }
    concat(other) {
        if (isIterable_1.isIterable(other)) {
            return new ItiririAsync(concat_1.concat(this.source, (function (e) { return tslib_1.__asyncGenerator(this, arguments, function* () { yield tslib_1.__await(yield* tslib_1.__asyncDelegator(tslib_1.__asyncValues(e))); }); })(other)));
        }
        return isAsyncIterable_1.isAsyncIterable(other) ?
            new ItiririAsync(concat_1.concat(this.source, other)) :
            new ItiririAsync(concat_1.concat(this.source, (function (e) { return tslib_1.__asyncGenerator(this, arguments, function* () { yield yield tslib_1.__await(e); }); })(other)));
    }
    prepend(other) {
        if (isIterable_1.isIterable(other)) {
            return new ItiririAsync(concat_1.concat((function (e) { return tslib_1.__asyncGenerator(this, arguments, function* () { yield tslib_1.__await(yield* tslib_1.__asyncDelegator(tslib_1.__asyncValues(e))); }); })(other), this.source));
        }
        return isAsyncIterable_1.isAsyncIterable(other) ?
            new ItiririAsync(concat_1.concat(other, this.source)) :
            new ItiririAsync(concat_1.concat((function (e) { return tslib_1.__asyncGenerator(this, arguments, function* () { yield yield tslib_1.__await(yield tslib_1.__await(e)); }); })(other), this.source));
    }
    nth(index) {
        return nth_1.nth(this.source, index);
    }
    indexOf(element, fromIndex = 0) {
        return indexOf_1.indexOf(this.source, (elem, idx) => idx >= fromIndex && elem === element);
    }
    findIndex(predicate) {
        return indexOf_1.indexOf(this.source, predicate);
    }
    lastIndexOf(element, fromIndex = 0) {
        return lastIndexOf_1.lastIndexOf(this.source, (elem, idx) => idx >= fromIndex && elem === element);
    }
    findLastIndex(predicate) {
        return lastIndexOf_1.lastIndexOf(this.source, predicate);
    }
    length(predicate = alwaysTrue()) {
        return length_1.length(filter_1.filter(this.source, predicate));
    }
    first() {
        return first_1.first(this.source);
    }
    find(predicate) {
        return first_1.first(filter_1.filter(this.source, predicate));
    }
    last() {
        return last_1.last(this.source);
    }
    findLast(predicate) {
        return last_1.last(filter_1.filter(this.source, predicate));
    }
    average(selector = element()) {
        return average_1.average(map_1.map(this.source, selector));
    }
    min(compareFn = comparer()) {
        return min_1.min(this.source, compareFn);
    }
    max(compareFn = comparer()) {
        return max_1.max(this.source, compareFn);
    }
    sum(selector = element()) {
        return sum_1.sum(map_1.map(this.source, selector));
    }
    awaitAll() {
        return awaitAll_1.awaitAll(this.source);
    }
    reduce(callback, initialValue) {
        return reduce_1.reduce(this.source, callback, initialValue);
    }
    every(predicate) {
        return indexOf_1.indexOf(this.source, (e, i) => !predicate(e, i)).then(index => index === -1);
    }
    some(predicate) {
        return indexOf_1.indexOf(this, predicate).then(index => index !== -1);
    }
    includes(element, fromIndex = 0) {
        return this.some((elem, idx) => idx >= fromIndex && elem === element);
    }
    filter(predicate) {
        return new ItiririAsync(filter_1.filter(this.source, predicate));
    }
    take(count) {
        return new ItiririAsync(take_1.take(this.source, count));
    }
    skip(count) {
        return new ItiririAsync(skip_1.skip(this.source, count));
    }
    slice(start, end) {
        return new ItiririAsync(slice_1.slice(this, start, end));
    }
    map(selector) {
        return new ItiririAsync(map_1.map(this.source, selector));
    }
    flat(selector = element()) {
        return new ItiririAsync(flat_1.flat(this.map(selector)));
    }
    distinct(selector = element()) {
        return new ItiririAsync(distinct_1.distinct(this.source, selector));
    }
    exclude(other, selector = element()) {
        return new ItiririAsync(exclude_1.exclude(this.source, other, selector));
    }
    intersect(other, selector = element()) {
        return new ItiririAsync(intersect_1.intersect(this.source, other, selector));
    }
    union(other, selector = element()) {
        return new ItiririAsync(distinct_1.distinct(concat_1.concat(this.source, other), selector));
    }
    join(other, leftKeySelector, rightKeySelector, joinSelector) {
        const iterator = join_1.join(this.source, other, leftKeySelector, rightKeySelector, joinSelector);
        return new ItiririAsync(iterator);
    }
    leftJoin(other, leftKeySelector, rightKeySelector, joinSelector) {
        const iterator = leftJoin_1.leftJoin(this.source, other, leftKeySelector, rightKeySelector, joinSelector);
        return new ItiririAsync(iterator);
    }
    groupJoin(other, leftKeySelector, rightKeySelector, joinSelector) {
        const iterator = groupJoin_1.groupJoin(this.source, other, leftKeySelector, rightKeySelector, joinSelector);
        return new ItiririAsync(iterator);
    }
}
function element() {
    return (e) => e;
}
function alwaysTrue() {
    return () => true;
}
function comparer() {
    return (a, b) => a === b ? 0 : (a > b ? 1 : -1);
}

//# sourceMappingURL=index.js.map
